@@ -61,11 +61,23 @@ _marker = indexer._marker
 LOG = log.getLogger(__name__)
 
 
+def _retry_on_exceptions(exc):
+    return (
+        pymysql and
+        isinstance(exc, exception.DBError) and
+        isinstance(exc.inner_exception, sqlalchemy.exc.InternalError) and
+        isinstance(exc.inner_exception.orig, pymysql.err.InternalError) and
+        (exc.inner_exception.orig.args[0] ==
+         pymysql.constants.ER.TABLE_DEF_CHANGED)
+    )
+
+
 def retry_on_deadlock(f):
     return oslo_db.api.wrap_db_retry(retry_on_deadlock=True,
                                      max_retries=20,
                                      retry_interval=0.1,
-                                     max_retry_interval=2)(f)
+                                     max_retry_interval=2,
+                                     exception_checker=_retry_on_exceptions)(f)
 
 
 class PerInstanceFacade(object):
@@ -397,6 +409,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
         try:
             with self.facade.independent_writer() as session:
+                engine = session.connection()
                 rt = self._get_resource_type(session, name)
 
                 with self.facade.writer_connection() as connection:
@@ -407,12 +420,22 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                             for attr in del_attributes:
                                 batch_op.drop_column(attr)
                             for attr in add_attributes:
-                                # TODO(sileht): When attr.required is True, we
-                                # have to pass a default. rest layer current
-                                # protect us, requied = True is not yet allowed
+                                server_default = attr.for_filling(
+                                    engine.dialect)
                                 batch_op.add_column(sqlalchemy.Column(
                                     attr.name, attr.satype,
-                                    nullable=not attr.required))
+                                    nullable=not attr.required,
+                                    server_default=server_default))
+
+                                # We have all rows filled now, we can remove
+                                # the server_default
+                                if server_default is not None:
+                                    batch_op.alter_column(
+                                        column_name=attr.name,
+                                        existing_type=attr.satype,
+                                        existing_server_default=server_default,
+                                        existing_nullable=not attr.required,
+                                        server_default=None)
 
                 rt.state = "active"
                 rt.updated_at = utils.utcnow()
@@ -640,12 +663,10 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         return apr
 
     @retry_on_deadlock
-    def create_metric(self, id, created_by_user_id, created_by_project_id,
-                      archive_policy_name,
+    def create_metric(self, id, creator, archive_policy_name,
                       name=None, unit=None, resource_id=None):
         m = Metric(id=id,
-                   created_by_user_id=created_by_user_id,
-                   created_by_project_id=created_by_project_id,
+                   creator=creator,
                    archive_policy_name=archive_policy_name,
                    name=name,
                    unit=unit,
@@ -653,10 +674,14 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         try:
             with self.facade.writer() as session:
                 session.add(m)
+        except exception.DBDuplicateEntry:
+            raise indexer.NamedMetricAlreadyExists(name)
         except exception.DBReferenceError as e:
             if (e.constraint ==
                'fk_metric_ap_name_ap_name'):
                 raise indexer.NoSuchArchivePolicy(archive_policy_name)
+            if e.constraint == 'fk_metric_resource_id_resource_id':
+                raise indexer.NoSuchResource(resource_id)
             raise
         return m
 
@@ -667,6 +692,8 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         sorts = sorts or []
         if ids is not None and not ids:
             return []
+        if names is not None and not names:
+            return []
         with self.facade.independent_reader() as session:
             q = session.query(Metric).filter(
                 Metric.status == status)
@@ -705,23 +732,25 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
     @retry_on_deadlock
     def create_resource(self, resource_type, id,
-                        created_by_user_id, created_by_project_id,
-                        user_id=None, project_id=None,
+                        creator, user_id=None, project_id=None,
                         started_at=None, ended_at=None, metrics=None,
+                        original_resource_id=None,
                         **kwargs):
         if (started_at is not None
            and ended_at is not None
            and started_at > ended_at):
             raise ValueError(
                 "Start timestamp cannot be after end timestamp")
+        if original_resource_id is None:
+            original_resource_id = str(id)
         with self.facade.writer() as session:
             resource_cls = self._resource_type_to_mappers(
                 session, resource_type)['resource']
             r = resource_cls(
                 id=id,
+                original_resource_id=original_resource_id,
                 type=resource_type,
-                created_by_user_id=created_by_user_id,
-                created_by_project_id=created_by_project_id,
+                creator=creator,
                 user_id=user_id,
                 project_id=project_id,
                 started_at=started_at,
@@ -824,10 +853,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                     update = session.query(Metric).filter(
                         Metric.id == value,
                         Metric.status == 'active',
-                        (Metric.created_by_user_id
-                         == r.created_by_user_id),
-                        (Metric.created_by_project_id
-                         == r.created_by_project_id),
+                        Metric.creator == r.creator,
                     ).update({"resource_id": r.id, "name": name})
                 except exception.DBDuplicateEntry:
                     raise indexer.NamedMetricAlreadyExists(name)
@@ -837,8 +863,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 unit = value.get('unit')
                 ap_name = value['archive_policy_name']
                 m = Metric(id=uuid.uuid4(),
-                           created_by_user_id=r.created_by_user_id,
-                           created_by_project_id=r.created_by_project_id,
+                           creator=r.creator,
                            archive_policy_name=ap_name,
                            name=name,
                            unit=unit,
@@ -1129,6 +1154,13 @@ class QueryTransformer(object):
         u"∧": sqlalchemy.and_,
     }
 
+    converters = (
+        (base.TimestampUTC, utils.to_datetime),
+        (types.String, six.text_type),
+        (types.Integer, int),
+        (types.Numeric, float),
+    )
+
     @classmethod
     def _handle_multiple_op(cls, engine, table, op, nodes):
         return op(*[
@@ -1140,8 +1172,8 @@ class QueryTransformer(object):
     def _handle_unary_op(cls, engine, table, op, node):
         return op(cls.build_filter(engine, table, node))
 
-    @staticmethod
-    def _handle_binary_op(engine, table, op, nodes):
+    @classmethod
+    def _handle_binary_op(cls, engine, table, op, nodes):
         try:
             field_name, value = list(nodes.items())[0]
         except Exception:
@@ -1155,6 +1187,24 @@ class QueryTransformer(object):
                 # weird results based on string comparison. It's useless and it
                 # does not work at all with seconds or anything. Just skip it.
                 raise exceptions.NotImplementedError
+        elif field_name == "created_by_user_id":
+            creator = getattr(table, "creator")
+            if op == operator.eq:
+                return creator.like("%s:%%" % value)
+            elif op == operator.ne:
+                return sqlalchemy.not_(creator.like("%s:%%" % value))
+            elif op == cls.binary_operators[u"like"]:
+                return creator.like("%s:%%" % value)
+            raise indexer.QueryValueError(value, field_name)
+        elif field_name == "created_by_project_id":
+            creator = getattr(table, "creator")
+            if op == operator.eq:
+                return creator.like("%%:%s" % value)
+            elif op == operator.ne:
+                return sqlalchemy.not_(creator.like("%%:%s" % value))
+            elif op == cls.binary_operators[u"like"]:
+                return creator.like("%%:%s" % value)
+            raise indexer.QueryValueError(value, field_name)
         else:
             try:
                 attr = getattr(table, field_name)
@@ -1167,29 +1217,17 @@ class QueryTransformer(object):
 
             # Convert value to the right type
             if value is not None:
-                converter = None
-
-                if isinstance(attr.type, base.PreciseTimestamp):
-                    converter = utils.to_timestamp
-                elif (isinstance(attr.type, sqlalchemy_utils.UUIDType)
-                      and not isinstance(value, uuid.UUID)):
-                    converter = utils.ResourceUUID
-                elif isinstance(attr.type, types.String):
-                    converter = six.text_type
-                elif isinstance(attr.type, types.Integer):
-                    converter = int
-                elif isinstance(attr.type, types.Numeric):
-                    converter = float
-
-                if converter:
-                    try:
-                        if isinstance(value, list):
-                            # we got a list for in_ operator
-                            value = [converter(v) for v in value]
-                        else:
-                            value = converter(value)
-                    except Exception:
-                        raise indexer.QueryValueError(value, field_name)
+                for klass, converter in cls.converters:
+                    if isinstance(attr.type, klass):
+                        try:
+                            if isinstance(value, list):
+                                # we got a list for in_ operator
+                                value = [converter(v) for v in value]
+                            else:
+                                value = converter(value)
+                        except Exception:
+                            raise indexer.QueryValueError(value, field_name)
+                        break
 
         return op(attr, value)
 
