@@ -18,7 +18,6 @@ import functools
 import itertools
 import uuid
 
-from concurrent import futures
 import jsonpatch
 from oslo_utils import dictutils
 from oslo_utils import strutils
@@ -163,8 +162,6 @@ RESOURCE_DEFAULT_PAGINATION = ['revision_start:asc',
 
 METRIC_DEFAULT_PAGINATION = ['id:asc']
 
-THREADS = utils.get_default_workers()
-
 
 def get_pagination_options(params, default):
     max_limit = pecan.request.conf.api.max_limit
@@ -1422,12 +1419,10 @@ class ResourcesMetricsMeasuresBatchController(rest.RestController):
         for metric in known_metrics:
             enforce("post measures", metric)
 
-        storage = pecan.request.storage.incoming
-        with futures.ThreadPoolExecutor(max_workers=THREADS) as executor:
-            list(executor.map(lambda x: storage.add_measures(*x),
-                              ((metric,
-                                body_by_rid[metric.resource_id][metric.name])
-                               for metric in known_metrics)))
+        pecan.request.storage.incoming.add_measures_batch(
+            dict((metric,
+                 body_by_rid[metric.resource_id][metric.name])
+                 for metric in known_metrics))
 
         pecan.response.status = 202
 
@@ -1456,11 +1451,9 @@ class MetricsMeasuresBatchController(rest.RestController):
         for metric in metrics:
             enforce("post measures", metric)
 
-        storage = pecan.request.storage.incoming
-        with futures.ThreadPoolExecutor(max_workers=THREADS) as executor:
-            list(executor.map(lambda x: storage.add_measures(*x),
-                              ((metric, body[metric.id]) for metric in
-                               metrics)))
+        pecan.request.storage.incoming.add_measures_batch(
+            dict((metric, body[metric.id]) for metric in
+                 metrics))
 
         pecan.response.status = 202
 
@@ -1623,9 +1616,9 @@ class AggregationController(rest.RestController):
         except storage.MetricUnaggregatable as e:
             abort(400, ("One of the metrics being aggregated doesn't have "
                         "matching granularity: %s") % str(e))
-        except storage.MetricDoesNotExist as e:
-            abort(404, e)
-        except storage.AggregationDoesNotExist as e:
+        except (storage.MetricDoesNotExist,
+                storage.GranularityDoesNotExist,
+                storage.AggregationDoesNotExist) as e:
             abort(404, e)
 
     @pecan.expose('json')
