@@ -19,6 +19,7 @@ try:
     import asyncio
 except ImportError:
     import trollius as asyncio
+from oslo_config import cfg
 from oslo_log import log
 import six
 
@@ -40,13 +41,12 @@ class Stats(object):
         try:
             self.indexer.create_resource('generic',
                                          self.conf.statsd.resource_id,
-                                         self.conf.statsd.user_id,
-                                         self.conf.statsd.project_id)
+                                         self.conf.statsd.creator)
         except indexer.ResourceAlreadyExists:
-            LOG.info("Resource %s already exists"
-                     % self.conf.statsd.resource_id)
+            LOG.debug("Resource %s already exists",
+                      self.conf.statsd.resource_id)
         else:
-            LOG.info("Created resource %s" % self.conf.statsd.resource_id)
+            LOG.info("Created resource %s", self.conf.statsd.resource_id)
         self.gauges = {}
         self.counters = {}
         self.times = {}
@@ -64,14 +64,14 @@ class Stats(object):
                     "Invalid sampling for ms: `%d`, should be none"
                     % sampling)
             self.times[metric_name] = storage.Measure(
-                utils.utcnow(), value)
+                utils.dt_in_unix_ns(utils.utcnow()), value)
         elif metric_type == "g":
             if sampling is not None:
                 raise ValueError(
                     "Invalid sampling for g: `%d`, should be none"
                     % sampling)
             self.gauges[metric_name] = storage.Measure(
-                utils.utcnow(), value)
+                utils.dt_in_unix_ns(utils.utcnow()), value)
         elif metric_type == "c":
             sampling = 1 if sampling is None else sampling
             if metric_name in self.counters:
@@ -79,7 +79,7 @@ class Stats(object):
             else:
                 current_value = 0
             self.counters[metric_name] = storage.Measure(
-                utils.utcnow(),
+                utils.dt_in_unix_ns(utils.utcnow()),
                 current_value + (value * (1 / sampling)))
         # TODO(jd) Support "set" type
         # elif metric_type == "s":
@@ -106,15 +106,14 @@ class Stats(object):
                     ap_name = self._get_archive_policy_name(metric_name)
                     metric = self.indexer.create_metric(
                         uuid.uuid4(),
-                        self.conf.statsd.user_id,
-                        self.conf.statsd.project_id,
+                        self.conf.statsd.creator,
                         archive_policy_name=ap_name,
                         name=metric_name,
                         resource_id=self.conf.statsd.resource_id)
-                self.storage.add_measures(metric, (measure,))
+                self.storage.incoming.add_measures(metric, (measure,))
             except Exception as e:
-                LOG.error("Unable to add measure %s: %s"
-                          % (metric_name, e))
+                LOG.error("Unable to add measure %s: %s",
+                          metric_name, e)
 
         self.reset()
 
@@ -135,11 +134,11 @@ class StatsdServer(object):
         pass
 
     def datagram_received(self, data, addr):
-        LOG.debug("Received data `%r' from %s" % (data, addr))
+        LOG.debug("Received data `%r' from %s", data, addr)
         try:
             messages = [m for m in data.decode().split("\n") if m]
         except Exception as e:
-            LOG.error("Unable to decode datagram: %s" % e)
+            LOG.error("Unable to decode datagram: %s", e)
             return
         for message in messages:
             metric = message.split("|")
@@ -149,7 +148,7 @@ class StatsdServer(object):
             elif len(metric) == 3:
                 metric_name, metric_type, sampling = metric
             else:
-                LOG.error("Invalid number of | in `%s'" % message)
+                LOG.error("Invalid number of | in `%s'", message)
                 continue
             sampling = float(sampling[1:]) if sampling is not None else None
             metric_name, metric_str_val = metric_name.split(':')
@@ -160,19 +159,22 @@ class StatsdServer(object):
                 self.stats.treat_metric(metric_name, metric_type,
                                         value, sampling)
             except Exception as e:
-                LOG.error("Unable to treat metric %s: %s" % (message, str(e)))
+                LOG.error("Unable to treat metric %s: %s", message, str(e))
 
 
 def start():
     conf = service.prepare_service()
 
+    if conf.statsd.resource_id is None:
+        raise cfg.RequiredOptError("resource_id", cfg.OptGroup("statsd"))
+
     stats = Stats(conf)
 
     loop = asyncio.get_event_loop()
     # TODO(jd) Add TCP support
     listen = loop.create_datagram_endpoint(
-        # TODO(jd) Add config options for host/port
-        lambda: StatsdServer(stats), local_addr=('0.0.0.0', 8125))
+        lambda: StatsdServer(stats),
+        local_addr=(conf.statsd.host, conf.statsd.port))
 
     def _flush():
         loop.call_later(conf.statsd.flush_delay, _flush)
@@ -181,6 +183,9 @@ def start():
     loop.call_later(conf.statsd.flush_delay, _flush)
     transport, protocol = loop.run_until_complete(listen)
 
+    LOG.info("Started on %s:%d", conf.statsd.host, conf.statsd.port)
+    LOG.info("Flush delay: %d seconds", conf.statsd.flush_delay)
+
     try:
         loop.run_forever()
     except KeyboardInterrupt:
