@@ -20,7 +20,6 @@ import math
 import fixtures
 from oslo_utils import timeutils
 from oslotest import base
-# TODO(jd) We shouldn't use pandas here
 import pandas
 import six
 
@@ -29,7 +28,7 @@ from gnocchi import carbonara
 
 class TestBoundTimeSerie(base.BaseTestCase):
     def test_benchmark(self):
-        self.useFixture(fixtures.Timeout(120, gentle=True))
+        self.useFixture(fixtures.Timeout(300, gentle=True))
         carbonara.BoundTimeSerie.benchmark()
 
     @staticmethod
@@ -68,8 +67,8 @@ class TestBoundTimeSerie(base.BaseTestCase):
     def test_block_size_unordered(self):
         ts = carbonara.BoundTimeSerie.from_data(
             [datetime.datetime(2014, 1, 1, 12, 0, 0),
-             datetime.datetime(2014, 1, 1, 12, 0, 9),
-             datetime.datetime(2014, 1, 1, 12, 0, 5)],
+             datetime.datetime(2014, 1, 1, 12, 0, 5),
+             datetime.datetime(2014, 1, 1, 12, 0, 9)],
             [10, 5, 23],
             block_size='5s')
         self.assertEqual(2, len(ts))
@@ -80,9 +79,8 @@ class TestBoundTimeSerie(base.BaseTestCase):
     def test_duplicate_timestamps(self):
         ts = carbonara.BoundTimeSerie.from_data(
             [datetime.datetime(2014, 1, 1, 12, 0, 0),
-             datetime.datetime(2014, 1, 1, 12, 0, 9),
              datetime.datetime(2014, 1, 1, 12, 0, 9)],
-            [10, 5, 23])
+            [10, 23])
         self.assertEqual(2, len(ts))
         self.assertEqual(10.0, ts[0])
         self.assertEqual(23.0, ts[1])
@@ -116,7 +114,7 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             [3, 5, 6])
 
     def test_benchmark(self):
-        self.useFixture(fixtures.Timeout(120, gentle=True))
+        self.useFixture(fixtures.Timeout(300, gentle=True))
         carbonara.AggregatedTimeSerie.benchmark()
 
     def test_fetch_basic(self):
@@ -158,13 +156,6 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         self.assertRaises(carbonara.BeforeEpochError,
                           ts.group_serie, 60)
 
-    def test_bad_percentile(self):
-        for bad_percentile in ('0pct', '100pct', '-1pct', '123pct'):
-            self.assertRaises(carbonara.UnknownAggregationMethod,
-                              carbonara.AggregatedTimeSerie,
-                              sampling='1Min',
-                              aggregation_method=bad_percentile)
-
     @staticmethod
     def _resample(ts, sampling, agg, max_size=None):
         grouped = ts.group_serie(sampling)
@@ -208,6 +199,47 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         self.assertEqual(5.9000000000000004,
                          ts[datetime.datetime(2014, 1, 1, 12, 0, 0)])
 
+    def _do_test_aggregation(self, name, v1, v2):
+        ts = carbonara.TimeSerie.from_tuples(
+            [(datetime.datetime(2014, 1, 1, 12, 0, 0), 3),
+             (datetime.datetime(2014, 1, 1, 12, 0, 4), 6),
+             (datetime.datetime(2014, 1, 1, 12, 0, 9), 5),
+             (datetime.datetime(2014, 1, 1, 12, 1, 4), 8),
+             (datetime.datetime(2014, 1, 1, 12, 1, 6), 9)])
+        ts = self._resample(ts, 60, name)
+
+        self.assertEqual(2, len(ts))
+        self.assertEqual(v1, ts[datetime.datetime(2014, 1, 1, 12, 0, 0)])
+        self.assertEqual(v2, ts[datetime.datetime(2014, 1, 1, 12, 1, 0)])
+
+    def test_aggregation_first(self):
+        self._do_test_aggregation('first', 3, 8)
+
+    def test_aggregation_last(self):
+        self._do_test_aggregation('last', 5, 9)
+
+    def test_aggregation_count(self):
+        self._do_test_aggregation('count', 3, 2)
+
+    def test_aggregation_sum(self):
+        self._do_test_aggregation('sum', 14, 17)
+
+    def test_aggregation_mean(self):
+        self._do_test_aggregation('mean', 4.666666666666667, 8.5)
+
+    def test_aggregation_median(self):
+        self._do_test_aggregation('median', 5.0, 8.5)
+
+    def test_aggregation_min(self):
+        self._do_test_aggregation('min', 3, 8)
+
+    def test_aggregation_max(self):
+        self._do_test_aggregation('max', 6, 9)
+
+    def test_aggregation_std(self):
+        self._do_test_aggregation('std', 1.5275252316519465,
+                                  0.70710678118654757)
+
     def test_different_length_in_timestamps_and_data(self):
         self.assertRaises(ValueError,
                           carbonara.AggregatedTimeSerie.from_data,
@@ -362,14 +394,30 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             aggregation='mean', needed_percent_of_overlap=80.0)
 
         self.assertEqual([
-            (pandas.Timestamp('2014-01-01 12:01:00'), 60.0, 3.0),
-            (pandas.Timestamp('2014-01-01 12:02:00'), 60.0, 3.0),
-            (pandas.Timestamp('2014-01-01 12:03:00'), 60.0, 4.0),
-            (pandas.Timestamp('2014-01-01 12:04:00'), 60.0, 4.0),
-            (pandas.Timestamp('2014-01-01 12:05:00'), 60.0, 3.0),
-            (pandas.Timestamp('2014-01-01 12:06:00'), 60.0, 5.0),
-            (pandas.Timestamp('2014-01-01 12:07:00'), 60.0, 10.0),
-            (pandas.Timestamp('2014-01-01 12:09:00'), 60.0, 2.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 1, 0
+            ), 60.0, 3.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 2, 0
+            ), 60.0, 3.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 3, 0
+            ), 60.0, 4.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 4, 0
+            ), 60.0, 4.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 5, 0
+            ), 60.0, 3.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 6, 0
+            ), 60.0, 5.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 7, 0
+            ), 60.0, 10.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 9, 0
+            ), 60.0, 2.0),
         ], output)
 
     def test_aggregated_different_archive_overlap_edge_missing1(self):
@@ -406,10 +454,18 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             tsc1['return'], tsc2['return']], aggregation='sum')
 
         self.assertEqual([
-            (pandas.Timestamp('2014-01-01 12:03:00'), 60.0, 33.0),
-            (pandas.Timestamp('2014-01-01 12:04:00'), 60.0, 5.0),
-            (pandas.Timestamp('2014-01-01 12:05:00'), 60.0, 18.0),
-            (pandas.Timestamp('2014-01-01 12:06:00'), 60.0, 19.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 3, 0
+            ), 60.0, 33.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 4, 0
+            ), 60.0, 5.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 5, 0
+            ), 60.0, 18.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 6, 0
+            ), 60.0, 19.0),
         ], output)
 
     def test_aggregated_different_archive_overlap_edge_missing2(self):
@@ -432,7 +488,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         output = carbonara.AggregatedTimeSerie.aggregated(
             [tsc1['return'], tsc2['return']], aggregation='mean')
         self.assertEqual([
-            (pandas.Timestamp('2014-01-01 12:03:00'), 60.0, 4.0),
+            (datetime.datetime(
+                2014, 1, 1, 12, 3, 0
+            ), 60.0, 4.0),
         ], output)
 
     def test_fetch(self):
@@ -486,6 +544,126 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             (datetime.datetime(2014, 1, 1, 12, 6), 60.0, 4.0)
         ], ts['return'].fetch(datetime.datetime(2014, 1, 1, 12, 0, 0)))
 
+    def test_aggregated_some_overlap_with_fill_zero(self):
+        tsc1 = {'sampling': 60, 'size': 10, 'agg': 'mean'}
+        tsb1 = carbonara.BoundTimeSerie(block_size=tsc1['sampling'])
+        tsc2 = {'sampling': 60, 'size': 10, 'agg': 'mean'}
+        tsb2 = carbonara.BoundTimeSerie(block_size=tsc2['sampling'])
+
+        tsb1.set_values([
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 9),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 1),
+            (datetime.datetime(2014, 1, 1, 12, 5, 0), 2),
+            (datetime.datetime(2014, 1, 1, 12, 6, 0), 7),
+            (datetime.datetime(2014, 1, 1, 12, 7, 0), 5),
+            (datetime.datetime(2014, 1, 1, 12, 8, 0), 3),
+        ], before_truncate_callback=functools.partial(
+            self._resample_and_merge, agg_dict=tsc1))
+
+        tsb2.set_values([
+            (datetime.datetime(2014, 1, 1, 12, 0, 0), 6),
+            (datetime.datetime(2014, 1, 1, 12, 1, 0), 2),
+            (datetime.datetime(2014, 1, 1, 12, 2, 0), 13),
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 24),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 4),
+            (datetime.datetime(2014, 1, 1, 12, 5, 0), 16),
+            (datetime.datetime(2014, 1, 1, 12, 6, 0), 12),
+        ], before_truncate_callback=functools.partial(
+            self._resample_and_merge, agg_dict=tsc2))
+
+        output = carbonara.AggregatedTimeSerie.aggregated([
+            tsc1['return'], tsc2['return']], aggregation='mean', fill=0)
+
+        self.assertEqual([
+            (datetime.datetime(2014, 1, 1, 12, 0, 0), 60.0, 3.0),
+            (datetime.datetime(2014, 1, 1, 12, 1, 0), 60.0, 1.0),
+            (datetime.datetime(2014, 1, 1, 12, 2, 0), 60.0, 6.5),
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 60.0, 16.5),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 60.0, 2.5),
+            (datetime.datetime(2014, 1, 1, 12, 5, 0), 60.0, 9.0),
+            (datetime.datetime(2014, 1, 1, 12, 6, 0), 60.0, 9.5),
+            (datetime.datetime(2014, 1, 1, 12, 7, 0), 60.0, 2.5),
+            (datetime.datetime(2014, 1, 1, 12, 8, 0), 60.0, 1.5),
+        ], output)
+
+    def test_aggregated_some_overlap_with_fill_null(self):
+        tsc1 = {'sampling': 60, 'size': 10, 'agg': 'mean'}
+        tsb1 = carbonara.BoundTimeSerie(block_size=tsc1['sampling'])
+        tsc2 = {'sampling': 60, 'size': 10, 'agg': 'mean'}
+        tsb2 = carbonara.BoundTimeSerie(block_size=tsc2['sampling'])
+
+        tsb1.set_values([
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 9),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 1),
+            (datetime.datetime(2014, 1, 1, 12, 5, 0), 2),
+            (datetime.datetime(2014, 1, 1, 12, 6, 0), 7),
+            (datetime.datetime(2014, 1, 1, 12, 7, 0), 5),
+            (datetime.datetime(2014, 1, 1, 12, 8, 0), 3),
+        ], before_truncate_callback=functools.partial(
+            self._resample_and_merge, agg_dict=tsc1))
+
+        tsb2.set_values([
+            (datetime.datetime(2014, 1, 1, 12, 0, 0), 6),
+            (datetime.datetime(2014, 1, 1, 12, 1, 0), 2),
+            (datetime.datetime(2014, 1, 1, 12, 2, 0), 13),
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 24),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 4),
+            (datetime.datetime(2014, 1, 1, 12, 5, 0), 16),
+            (datetime.datetime(2014, 1, 1, 12, 6, 0), 12),
+        ], before_truncate_callback=functools.partial(
+            self._resample_and_merge, agg_dict=tsc2))
+
+        output = carbonara.AggregatedTimeSerie.aggregated([
+            tsc1['return'], tsc2['return']], aggregation='mean', fill='null')
+
+        self.assertEqual([
+            (datetime.datetime(2014, 1, 1, 12, 0, 0), 60.0, 6.0),
+            (datetime.datetime(2014, 1, 1, 12, 1, 0), 60.0, 2.0),
+            (datetime.datetime(2014, 1, 1, 12, 2, 0), 60.0, 13.0),
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 60.0, 16.5),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 60.0, 2.5),
+            (datetime.datetime(2014, 1, 1, 12, 5, 0), 60.0, 9.0),
+            (datetime.datetime(2014, 1, 1, 12, 6, 0), 60.0, 9.5),
+            (datetime.datetime(2014, 1, 1, 12, 7, 0), 60.0, 5.0),
+            (datetime.datetime(2014, 1, 1, 12, 8, 0), 60.0, 3.0),
+        ], output)
+
+    def test_aggregate_no_points_with_fill_zero(self):
+        tsc1 = {'sampling': 60, 'size': 10, 'agg': 'mean'}
+        tsb1 = carbonara.BoundTimeSerie(block_size=tsc1['sampling'])
+        tsc2 = {'sampling': 60, 'size': 10, 'agg': 'mean'}
+        tsb2 = carbonara.BoundTimeSerie(block_size=tsc2['sampling'])
+
+        tsb1.set_values([
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 9),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 1),
+            (datetime.datetime(2014, 1, 1, 12, 7, 0), 5),
+            (datetime.datetime(2014, 1, 1, 12, 8, 0), 3),
+        ], before_truncate_callback=functools.partial(
+            self._resample_and_merge, agg_dict=tsc1))
+
+        tsb2.set_values([
+            (datetime.datetime(2014, 1, 1, 12, 0, 0), 6),
+            (datetime.datetime(2014, 1, 1, 12, 1, 0), 2),
+            (datetime.datetime(2014, 1, 1, 12, 2, 0), 13),
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 24),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 4),
+        ], before_truncate_callback=functools.partial(
+            self._resample_and_merge, agg_dict=tsc2))
+
+        output = carbonara.AggregatedTimeSerie.aggregated([
+            tsc1['return'], tsc2['return']], aggregation='mean', fill=0)
+
+        self.assertEqual([
+            (datetime.datetime(2014, 1, 1, 12, 0, 0), 60.0, 3.0),
+            (datetime.datetime(2014, 1, 1, 12, 1, 0), 60.0, 1.0),
+            (datetime.datetime(2014, 1, 1, 12, 2, 0), 60.0, 6.5),
+            (datetime.datetime(2014, 1, 1, 12, 3, 0), 60.0, 16.5),
+            (datetime.datetime(2014, 1, 1, 12, 4, 0), 60.0, 2.5),
+            (datetime.datetime(2014, 1, 1, 12, 7, 0), 60.0, 2.5),
+            (datetime.datetime(2014, 1, 1, 12, 8, 0), 60.0, 1.5),
+        ], output)
+
     def test_fetch_agg_pct(self):
         ts = {'sampling': 1, 'size': 3600 * 24, 'agg': '90pct'}
         tsb = carbonara.BoundTimeSerie(block_size=ts['sampling'])
@@ -498,10 +676,12 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         result = ts['return'].fetch(datetime.datetime(2014, 1, 1, 12, 0, 0))
         reference = [
-            (pandas.Timestamp('2014-01-01 12:00:00'),
-             1.0, 3.9),
-            (pandas.Timestamp('2014-01-01 12:00:02'),
-             1.0, 4)
+            (datetime.datetime(
+                2014, 1, 1, 12, 0, 0
+            ), 1.0, 3.9),
+            (datetime.datetime(
+                2014, 1, 1, 12, 0, 2
+            ), 1.0, 4)
         ]
 
         self.assertEqual(len(reference), len(result))
@@ -518,10 +698,12 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         result = ts['return'].fetch(datetime.datetime(2014, 1, 1, 12, 0, 0))
         reference = [
-            (pandas.Timestamp('2014-01-01 12:00:00'),
-             1.0, 3.9),
-            (pandas.Timestamp('2014-01-01 12:00:02'),
-             1.0, 99.4)
+            (datetime.datetime(
+                2014, 1, 1, 12, 0, 0
+            ), 1.0, 3.9),
+            (datetime.datetime(
+                2014, 1, 1, 12, 0, 2
+            ), 1.0, 99.4)
         ]
 
         self.assertEqual(len(reference), len(result))
@@ -572,10 +754,12 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (pandas.Timestamp('2014-01-01 12:01:00'),
-             60.0, 2.1213203435596424),
-            (pandas.Timestamp('2014-01-01 12:02:00'),
-             60.0, 9.8994949366116654),
+            (datetime.datetime(
+                2014, 1, 1, 12, 1, 0
+            ), 60.0, 2.1213203435596424),
+            (datetime.datetime(
+                2014, 1, 1, 12, 2, 0
+            ), 60.0, 9.8994949366116654),
         ], ts['return'].fetch(datetime.datetime(2014, 1, 1, 12, 0, 0)))
 
         tsb.set_values([(datetime.datetime(2014, 1, 1, 12, 2, 13), 110)],
@@ -583,10 +767,12 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (pandas.Timestamp('2014-01-01 12:01:00'),
-             60.0, 2.1213203435596424),
-            (pandas.Timestamp('2014-01-01 12:02:00'),
-             60.0, 59.304300012730948),
+            (datetime.datetime(
+                2014, 1, 1, 12, 1, 0
+            ), 60.0, 2.1213203435596424),
+            (datetime.datetime(
+                2014, 1, 1, 12, 2, 0
+            ), 60.0, 59.304300012730948),
         ], ts['return'].fetch(datetime.datetime(2014, 1, 1, 12, 0, 0)))
 
     def test_fetch_agg_max(self):
@@ -602,9 +788,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (pandas.Timestamp('2014-01-01 12:00:00'), 60.0, 3),
-            (pandas.Timestamp('2014-01-01 12:01:00'), 60.0, 7),
-            (pandas.Timestamp('2014-01-01 12:02:00'), 60.0, 15),
+            (datetime.datetime(
+                2014, 1, 1, 12, 0, 0
+            ), 60.0, 3),
+            (datetime.datetime(
+                2014, 1, 1, 12, 1, 0
+            ), 60.0, 7),
+            (datetime.datetime(
+                2014, 1, 1, 12, 2, 0
+            ), 60.0, 15),
         ], ts['return'].fetch(datetime.datetime(2014, 1, 1, 12, 0, 0)))
 
         tsb.set_values([(datetime.datetime(2014, 1, 1, 12, 2, 13), 110)],
@@ -612,9 +804,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (pandas.Timestamp('2014-01-01 12:00:00'), 60.0, 3),
-            (pandas.Timestamp('2014-01-01 12:01:00'), 60.0, 7),
-            (pandas.Timestamp('2014-01-01 12:02:00'), 60.0, 110),
+            (datetime.datetime(
+                2014, 1, 1, 12, 0, 0
+            ), 60.0, 3),
+            (datetime.datetime(
+                2014, 1, 1, 12, 1, 0
+            ), 60.0, 7),
+            (datetime.datetime(
+                2014, 1, 1, 12, 2, 0
+            ), 60.0, 110),
         ], ts['return'].fetch(datetime.datetime(2014, 1, 1, 12, 0, 0)))
 
     def test_serialize(self):
@@ -672,9 +870,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (pandas.Timestamp('2014-01-01 12:00:01'), 1.0, 1.5),
-                (pandas.Timestamp('2014-01-01 12:00:02'), 1.0, 3.5),
-                (pandas.Timestamp('2014-01-01 12:00:03'), 1.0, 2.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 1
+                ), 1.0, 1.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 2
+                ), 1.0, 3.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 3
+                ), 1.0, 2.5),
             ],
             ts['return'].fetch())
 
@@ -713,9 +917,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (pandas.Timestamp('2014-01-01 12:00:01'), 1.0, 1.5),
-                (pandas.Timestamp('2014-01-01 12:00:02'), 1.0, 3.5),
-                (pandas.Timestamp('2014-01-01 12:00:03'), 1.0, 2.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 1
+                ), 1.0, 1.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 2
+                ), 1.0, 3.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 3
+                ), 1.0, 2.5),
             ],
             ts['return'].fetch())
 
@@ -727,9 +937,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (pandas.Timestamp('2014-01-01 12:00:01'), 1.0, 1.5),
-                (pandas.Timestamp('2014-01-01 12:00:02'), 1.0, 3.5),
-                (pandas.Timestamp('2014-01-01 12:00:03'), 1.0, 2.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 1
+                ), 1.0, 1.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 2
+                ), 1.0, 3.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 3
+                ), 1.0, 2.5),
             ],
             ts['return'].fetch())
 
@@ -742,9 +958,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (pandas.Timestamp('2014-01-01 12:00:01'), 1.0, 1.5),
-                (pandas.Timestamp('2014-01-01 12:00:02'), 1.0, 3.5),
-                (pandas.Timestamp('2014-01-01 12:00:03'), 1.0, 3.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 1
+                ), 1.0, 1.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 2
+                ), 1.0, 3.5),
+                (datetime.datetime(
+                    2014, 1, 1, 12, 0, 3
+                ), 1.0, 3.5),
             ],
             ts['return'].fetch())
 
@@ -873,8 +1095,12 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             [tsc1['return'], tsc2['return']], aggregation="sum")
 
         self.assertEqual([
-            (pandas.Timestamp('2015-12-03 13:21:15'), 1.0, 11.0),
-            (pandas.Timestamp('2015-12-03 13:22:15'), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 21, 15
+            ), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 22, 15
+            ), 1.0, 11.0),
         ], output)
 
         dtfrom = datetime.datetime(2015, 12, 3, 13, 17, 0)
@@ -886,12 +1112,24 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             aggregation="sum", needed_percent_of_overlap=0)
 
         self.assertEqual([
-            (pandas.Timestamp('2015-12-03 13:19:15'), 1.0, 1.0),
-            (pandas.Timestamp('2015-12-03 13:20:15'), 1.0, 1.0),
-            (pandas.Timestamp('2015-12-03 13:21:15'), 1.0, 11.0),
-            (pandas.Timestamp('2015-12-03 13:22:15'), 1.0, 11.0),
-            (pandas.Timestamp('2015-12-03 13:23:15'), 1.0, 10.0),
-            (pandas.Timestamp('2015-12-03 13:24:15'), 1.0, 10.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 19, 15
+            ), 1.0, 1.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 20, 15
+            ), 1.0, 1.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 21, 15
+            ), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 22, 15
+            ), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 23, 15
+            ), 1.0, 10.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 24, 15
+            ), 1.0, 10.0),
         ], output)
 
         # By default we require 100% of point that overlap
@@ -911,10 +1149,18 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             aggregation="sum",
             needed_percent_of_overlap=50.0)
         self.assertEqual([
-            (pandas.Timestamp('2015-12-03 13:19:15'), 1.0, 1.0),
-            (pandas.Timestamp('2015-12-03 13:20:15'), 1.0, 1.0),
-            (pandas.Timestamp('2015-12-03 13:21:15'), 1.0, 11.0),
-            (pandas.Timestamp('2015-12-03 13:22:15'), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 19, 15
+            ), 1.0, 1.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 20, 15
+            ), 1.0, 1.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 21, 15
+            ), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 22, 15
+            ), 1.0, 11.0),
         ], output)
 
         output = carbonara.AggregatedTimeSerie.aggregated(
@@ -922,10 +1168,18 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             aggregation="sum",
             needed_percent_of_overlap=50.0)
         self.assertEqual([
-            (pandas.Timestamp('2015-12-03 13:21:15'), 1.0, 11.0),
-            (pandas.Timestamp('2015-12-03 13:22:15'), 1.0, 11.0),
-            (pandas.Timestamp('2015-12-03 13:23:15'), 1.0, 10.0),
-            (pandas.Timestamp('2015-12-03 13:24:15'), 1.0, 10.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 21, 15
+            ), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 22, 15
+            ), 1.0, 11.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 23, 15
+            ), 1.0, 10.0),
+            (datetime.datetime(
+                2015, 12, 3, 13, 24, 15
+            ), 1.0, 10.0),
         ], output)
 
     def test_split_key(self):
@@ -949,66 +1203,6 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertGreaterEqual(key, pandas.Timestamp(0))
 
-    def test_split_key_cmp(self):
-        dt1 = datetime.datetime(2015, 1, 1, 15, 3)
-        dt1_1 = datetime.datetime(2015, 1, 1, 15, 3)
-        dt2 = datetime.datetime(2015, 1, 5, 15, 3)
-        td = 60
-
-        self.assertEqual(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td))
-        self.assertEqual(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1_1, td))
-        self.assertNotEqual(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
-
-        self.assertLess(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
-        self.assertLessEqual(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td))
-
-        self.assertGreater(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td),
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td))
-        self.assertGreaterEqual(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td),
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
-
-    def test_split_key_cmp_negative(self):
-        dt1 = datetime.datetime(2015, 1, 1, 15, 3)
-        dt1_1 = datetime.datetime(2015, 1, 1, 15, 3)
-        dt2 = datetime.datetime(2015, 1, 5, 15, 3)
-        td = 60
-
-        self.assertFalse(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) !=
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td))
-        self.assertFalse(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) !=
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1_1, td))
-        self.assertFalse(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) ==
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
-
-        self.assertFalse(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) >=
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
-        self.assertFalse(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) >
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td))
-
-        self.assertFalse(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td) <=
-            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td))
-        self.assertFalse(
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td) <
-            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
-
     def test_split_key_next(self):
         self.assertEqual(
             datetime.datetime(2015, 3, 6),
@@ -1063,3 +1257,19 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                              sampling=agg.sampling,
                              max_size=agg.max_size,
                              aggregation_method=agg.aggregation_method))
+
+    def test_resample(self):
+        ts = carbonara.TimeSerie.from_data(
+            [datetime.datetime(2014, 1, 1, 12, 0, 0),
+             datetime.datetime(2014, 1, 1, 12, 0, 4),
+             datetime.datetime(2014, 1, 1, 12, 0, 9),
+             datetime.datetime(2014, 1, 1, 12, 0, 11),
+             datetime.datetime(2014, 1, 1, 12, 0, 12)],
+            [3, 5, 6, 2, 4])
+        agg_ts = self._resample(ts, 5, 'mean')
+        self.assertEqual(3, len(agg_ts))
+
+        agg_ts = agg_ts.resample(10)
+        self.assertEqual(2, len(agg_ts))
+        self.assertEqual(5, agg_ts[0])
+        self.assertEqual(3, agg_ts[1])
